/*
  Copyright (C) 2010 Alexander Vorobiev.  All rights reserved.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  See file LICENSE.txt for further informations on licensing terms.
*/



#include "RailwayController.h"
#include "WProgram.h"



void TRailwayController::Setup(TRailwayHardware* hw) {
	m_pHW = hw;
	
	QueueClear();
	m_Command = 0;	//   

	//  
	Serial.begin(9600); 
}

void TRailwayController::LoopCall(void) {
	if( m_Command ) {
		//  
		ContinueCommandExecution();
	}
	if( ! m_Command ) {
		//    
		while( !TryNextCommand() ) {}
	}
}

//     
bool TRailwayController::IsExecuting(void) {
	if( m_Command != 0 ) return true; //   
	if( m_QueueReadPtr != m_QueueWritePtr ) return true; //  -  
	return false;
}






//     
int TRailwayController::BytesAtQueue(void) {
	if( m_QueueWritePtr >= m_QueueReadPtr ) return m_QueueWritePtr - m_QueueReadPtr;
	return RWCTRL_QUEUE_SIZE - m_QueueReadPtr + m_QueueWritePtr;
}


//     
int TRailwayController::FreeAtQueue(void) {
	return RWCTRL_QUEUE_SIZE - BytesAtQueue() - 1;//        
}


//   ( !)
void TRailwayController::QueueGet(uint8_t* pd) {
	*pd = m_Queue[m_QueueReadPtr];
	m_QueueReadPtr++;
	if( m_QueueReadPtr == RWCTRL_QUEUE_SIZE ) m_QueueReadPtr = 0;
}

//   ( !)
void TRailwayController::QueueGet(uint16_t* pd) {
	//   ()
	*pd = (uint16_t) (m_Queue[m_QueueReadPtr]);
	m_QueueReadPtr++;
	if( m_QueueReadPtr == RWCTRL_QUEUE_SIZE ) m_QueueReadPtr = 0;
	(*pd) <<= 8;
	//   ()
	(*pd) |= (uint16_t) (m_Queue[m_QueueReadPtr]);
	m_QueueReadPtr++;
	if( m_QueueReadPtr == RWCTRL_QUEUE_SIZE ) m_QueueReadPtr = 0;
}

//   ( !)
void TRailwayController::QueuePut(uint8_t d) {
	m_Queue[m_QueueWritePtr] = d;
	m_QueueWritePtr++;
	if( m_QueueWritePtr == RWCTRL_QUEUE_SIZE ) m_QueueWritePtr = 0;
}

//   ( !)
void TRailwayController::QueuePut(uint16_t d) {
	//   ()
	m_Queue[m_QueueWritePtr] = (uint8_t)(d >> 8);
	m_QueueWritePtr++;
	if( m_QueueWritePtr == RWCTRL_QUEUE_SIZE ) m_QueueWritePtr = 0;
	//   ()
	m_Queue[m_QueueWritePtr] = (uint8_t)d;
	m_QueueWritePtr++;
	if( m_QueueWritePtr == RWCTRL_QUEUE_SIZE ) m_QueueWritePtr = 0;
}


//  
void TRailwayController::QueueClear(void) {
	m_QueueReadPtr = 0;
	m_QueueWritePtr = 0;
}



// ----------------   -----------------

// != 0 !!!!
const uint8_t	CMD_DELAY			= 1;
const uint8_t	CMD_LIGHTS_RED		= 2;
const uint8_t	CMD_LIGHTS_GREEN	= 3;
const uint8_t	CMD_WAITSENS_ON		= 4;
const uint8_t	CMD_WAITSENS_OFF	= 5;
const uint8_t	CMD_SWITCH_LEFT		= 6;
const uint8_t	CMD_SWITCH_RIGHT	= 7;
const uint8_t	CMD_TRAIN_RUN		= 8;
const uint8_t	CMD_PLAY_SOUND		= 9;


//  (ms)
void TRailwayController::DelayMs(uint16_t ms) {
	//      3 
	if( FreeAtQueue() < 3 ) return;
	//  
	QueuePut( CMD_DELAY );
	//  
	QueuePut( ms );
}


//   
bool TRailwayController::ExecDelayMs(void) {
	//   
	if( BytesAtQueue() < 2 ) return false; //    -   
	QueueGet( &m_ArgWord );
	// 
	m_Command = CMD_DELAY;
	m_DelayMsStart = millis();
	return true;
}

//   
void TRailwayController::ContinueDelayMs(void) {
	unsigned long tm = millis();
	if( tm - m_DelayMsStart < m_ArgWord ) return; //   
	//  
	m_Command = 0;
}



//  
// lt -   LT..
void TRailwayController::LightsRed(uint16_t lt) {
	//      3 
	if( FreeAtQueue() < 3 ) return;
	//  
	QueuePut( CMD_LIGHTS_RED );
	//  
	QueuePut( lt );
}


//   -  
//    false .    
bool TRailwayController::ExecLightsRed(void) {
	//   
	if( BytesAtQueue() < 2 ) return false;
	uint16_t lt;
	QueueGet( &lt );
	m_pHW->m_LightsCtrl.Red( lt );
	return false;
}



//  
// lt -   LT..
void TRailwayController::LightsGreen(uint16_t lt) {
	//      3 
	if( FreeAtQueue() < 3 ) return;
	//  
	QueuePut( CMD_LIGHTS_GREEN );
	//  
	QueuePut( lt );
}


//   -  
//    false .    
bool TRailwayController::ExecLightsGreen(void) {
	//   
	if( BytesAtQueue() < 2 ) return false;
	uint16_t lt;
	QueueGet( &lt );
	m_pHW->m_LightsCtrl.Green( lt );
	return false;
}


//    
void TRailwayController::WaitCrossOn(uint8_t sn) {
	//      2 
	if( FreeAtQueue() < 2 ) return;
	//  
	QueuePut( CMD_WAITSENS_ON );
	//  
	QueuePut( sn );
}


//   -     
bool TRailwayController::ExecWaitCrossOn(void) {
	//   
	if( !BytesAtQueue() ) return false;
	QueueGet( &m_ArgByte );
	//   
	if( m_pHW->m_CrossSens.m_CrossOn[ m_ArgByte ] ) {
		//   -  
		return false; //    
	}
	//    -  
	m_Command = CMD_WAITSENS_ON;
	return true;
}


//     
void TRailwayController::ContinueWaitCrossOn(void) {
	if( m_pHW->m_CrossSens.m_CrossOn[ m_ArgByte ] ) {
		//  
		m_Command = 0;
	}
}



//    
void TRailwayController::WaitCrossOff(uint8_t sn) {
	//      2 
	if( FreeAtQueue() < 2 ) return;
	//  
	QueuePut( CMD_WAITSENS_OFF );
	//  
	QueuePut( sn );
}


//   -     
bool TRailwayController::ExecWaitCrossOff(void) {
	//   
	if( !BytesAtQueue() ) return false;
	QueueGet( &m_ArgByte );
	//   
	if( m_pHW->m_CrossSens.m_CrossOff[ m_ArgByte ] ) {
		//   -  
		return false; //    
	}
	//    -  
	m_Command = CMD_WAITSENS_OFF;
	return true;
}


//     
void TRailwayController::ContinueWaitCrossOff(void) {
	if( m_pHW->m_CrossSens.m_CrossOff[ m_ArgByte ] ) {
		//  
		m_Command = 0;
	}
}



//    ( )
void TRailwayController::SwitchLeft(uint8_t wait/*0*/) {
	//      2 
	if( FreeAtQueue() < 2 ) return;
	//  
	QueuePut( CMD_SWITCH_LEFT );
	//  
	QueuePut( wait );
}


//   -   
bool TRailwayController::ExecSwitchLeft(void) {
	//   
	if( !BytesAtQueue() ) return false;
	QueueGet( &m_ArgByte );
	//     ?
	if( m_pHW->m_SwCtrl.m_Dir ) return false; //  -  
	// 
	m_pHW->m_SwCtrl.Left();
	if( !m_ArgByte ) return false; //    
	//   
	m_Command = CMD_SWITCH_LEFT;
	return true;
}


//   -   
void TRailwayController::ContinueSwitchLeft(void) {
	//   
	if( ! m_pHW->m_SwCtrl.m_Dir ) {
		// 
		m_Command = 0;
	}
}



//    ( )
void TRailwayController::SwitchRight(uint8_t wait/*0*/) {
	//      2 
	if( FreeAtQueue() < 2 ) return;
	//  
	QueuePut( CMD_SWITCH_RIGHT );
	//  
	QueuePut( wait );
}


//   -   
bool TRailwayController::ExecSwitchRight(void) {
	//   
	if( !BytesAtQueue() ) return false;
	QueueGet( &m_ArgByte );
	//     ?
	if( m_pHW->m_SwCtrl.m_Dir ) return false; //  -  
	// 
	m_pHW->m_SwCtrl.Right();
	if( !m_ArgByte ) return false; //    
	//   
	m_Command = CMD_SWITCH_RIGHT;
	return true;
}


//   -   
void TRailwayController::ContinueSwitchRight(void) {
	//   
	if( ! m_pHW->m_SwCtrl.m_Dir ) {
		// 
		m_Command = 0;
	}
}


//     
void TRailwayController::TrainRun(int8_t sp) {
	//      2 
	if( FreeAtQueue() < 2 ) return;
	//  
	QueuePut( CMD_TRAIN_RUN );
	//  
	QueuePut( (uint8_t)sp );
}


//   -  
bool TRailwayController::ExecTrainRun(void) {
	//   
	if( !BytesAtQueue() ) return false;
	uint8_t sp;
	QueueGet( &sp );
	// 
	m_pHW->m_TrainCtrl.Run( (int8_t)sp );
	return false; //    
}



//     (   SND%u)
void TRailwayController::PlaySound(uint8_t code) {
	//      2 
	if( FreeAtQueue() < 2 ) return;
	//  
	QueuePut( CMD_PLAY_SOUND );
	//  
	QueuePut( code );
}



//   -  
bool TRailwayController::ExecPlaySound(void) {
	//   
	if( !BytesAtQueue() ) return false;
	uint8_t code;
	QueueGet( &code );
	// 
	Serial.print("SND");
	Serial.print(code,DEC);
	Serial.print(";"); // 
	return false; //    
}




// --------------------------  ---------------------------

//   
void TRailwayController::ContinueCommandExecution(void) {
	switch( m_Command ) {
	case CMD_DELAY:
		ContinueDelayMs();
		break;
	case CMD_WAITSENS_ON:
		ContinueWaitCrossOn();
		break;
	case CMD_WAITSENS_OFF:
		ContinueWaitCrossOff();
		break;
	case CMD_SWITCH_LEFT:
		ContinueSwitchLeft();
		break;
	case CMD_SWITCH_RIGHT:
		ContinueSwitchRight();
		break;
	}
}

//   
//   -     
bool TRailwayController::TryNextCommand(void) {
	//       
	if( ! BytesAtQueue() ) return true; //  
	//  
	uint8_t command;
	QueueGet( &command );

	switch( command ) {
	case CMD_DELAY:
		return ExecDelayMs();
	case CMD_LIGHTS_RED:
		return ExecLightsRed();
	case CMD_LIGHTS_GREEN:
		return ExecLightsGreen();
	case CMD_WAITSENS_ON:
		return ExecWaitCrossOn();
	case CMD_WAITSENS_OFF:
		return ExecWaitCrossOff();
	case CMD_SWITCH_LEFT:
		return ExecSwitchLeft();
	case CMD_SWITCH_RIGHT:
		return ExecSwitchRight();
	case CMD_TRAIN_RUN:
		return ExecTrainRun();
	case CMD_PLAY_SOUND:
		return ExecPlaySound();
	default:
		return false; //   -   
	}

	return true;
}
