/*
  Copyright (C) 2010 Alexander Vorobiev.  All rights reserved.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  See file LICENSE.txt for further informations on licensing terms.
*/


#ifndef __RAILWAYCONTROLLER_H_
#define __RAILWAYCONTROLLER_H_

#include <inttypes.h>
#include <RailwayHardware.h>

//   
#define RWCTRL_QUEUE_SIZE		1024


class TRailwayController {
protected: // 
	TRailwayHardware*			m_pHW;		//   
	//  
	uint8_t						m_Queue[RWCTRL_QUEUE_SIZE];
	int							m_QueueReadPtr;	//    (  )
	int							m_QueueWritePtr;//    (  )
	// ----   ----
	uint8_t						m_Command;		//    (0- )
	uint16_t					m_ArgWord;		//    
	uint8_t						m_ArgByte;		//    
	//   
	unsigned long				m_DelayMsStart;	//    
protected: // 
	// -----    ------
	int BytesAtQueue(void);		//     
	int FreeAtQueue(void);		//     
	void QueueGet(uint8_t*);	//  
	void QueueGet(uint16_t*);	//  
	void QueuePut(uint8_t);		//  
	void QueuePut(uint16_t);	//  
	void QueueClear(void);		//  
	// -----   -----
	void ContinueCommandExecution(void);	//   
	bool TryNextCommand(void);				//   
	// 
	bool ExecDelayMs(void);		//   
	void ContinueDelayMs(void);	//   
	// 
	bool ExecLightsRed(void);	//   -  
	bool ExecLightsGreen(void);	//   -  
	//  
	bool ExecWaitCrossOn(void);	//   -     
	void ContinueWaitCrossOn(void);//     
	bool ExecWaitCrossOff(void);//   -     
	void ContinueWaitCrossOff(void);//     
	// 
	bool ExecSwitchLeft(void);	//   -   
	void ContinueSwitchLeft(void);//   -   
	bool ExecSwitchRight(void);	//   -   
	void ContinueSwitchRight(void);//   -   
	//  
	bool ExecTrainRun(void);	//   -  
	// 
	bool ExecPlaySound(void);	//   -  
public: // 
	void Setup(TRailwayHardware* hw);		//    setup
	void LoopCall(void);					//     loop
	bool IsExecuting(void);		//     
	// 
	void DelayMs(uint16_t);		//  (ms)
	//  
	void LightsRed(uint16_t);	//  
	void LightsGreen(uint16_t);	//  
	//  
	void WaitCrossOn(uint8_t);	//    
	void WaitCrossOff(uint8_t);	//    
	// 
	void SwitchLeft(uint8_t wait=0);//    ( )
	void SwitchRight(uint8_t wait=0);//    ( )
	//  
	void TrainRun(int8_t);		//     
	// 
	void PlaySound(uint8_t);	//     (   SND%u)
};



#endif