/*
  Copyright (C) 2010 Alexander Vorobiev.  All rights reserved.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  See file LICENSE.txt for further informations on licensing terms.
*/


#ifndef __RAILWAYHARDWARE_H_
#define __RAILWAYHARDWARE_H_

#include <inttypes.h>
#include <InOutExtender.h>
#include <AFMotor.h>

// -------------------------------------- TRailwayLightsControl ----------------------------------------------------
//  

//   ()
#define LT1		0x0001
#define LT2		0x0080
#define	LT3		0x0040
#define LT4		0x0020
#define	LT5		0x0010
#define	LT6		0x0008


class TRailwayLightsControl {
protected: // 
	TInOutExtenderS*		m_pIo;			//   
public: // 
	void Setup(TInOutExtenderS* io);
	void Red(uint16_t lt);					//  
	void Green(uint16_t lt);				//  
};


// -------------------------------------- TRailwayCrossSensors -----------------------------------------------------
//  

//  
#define SE1		0
#define	SE2		1
#define	SE3		2

class TRailwayCrossSensors {
public: // 
	uint8_t					m_CrossState[6];//   
	uint8_t					m_CrossOn[6];	//       
	uint8_t					m_CrossOff[6];	//    " "   
protected: // 
	uint8_t					m_Count;		//  
	TInOutExtenderS*		m_pIo;			//   
	unsigned long			m_PrevSwitchTime;//  ()   
	int						m_AnalogValue[6];//  
	uint8_t					m_CrossSignal[6];//   
	uint8_t					m_CrossCounter[6];//     
public: // 
	TRailwayCrossSensors(uint8_t count);
	void Setup(TInOutExtenderS* io);
	void LoopCall(void);					//  
};


// --------------------------------------- TRailwaySwitchControl ---------------------------------------------------
//  

class TRailwaySwitchControl {
public: // 
	uint8_t			m_Dir;					//    (=0 -  )
protected: // 
	AF_Stepper		m_SMotor;
	uint8_t			m_Step;					//  
	unsigned long	m_PrevTm;				//    
public: // 
	TRailwaySwitchControl(void);
	void Setup(void);						//    setup
	void LoopCall(void);					//  
	// 
	void Left(void);						//  
	void Right(void);						//  
};


// --------------------------------------- TRailwayTrainControl ----------------------------------------------------
//  

class TRailwayTrainControl {
protected: // 
	AF_DCMotor		m_Motor;
	int8_t			m_Speed;				//  /
	int8_t			m_TgSpeed;				//  
	unsigned long	m_PrevTm;				//    
public: // 
	TRailwayTrainControl(void);
	void Setup(void);						//    setup
	void LoopCall(void);					//  
	void Run(int8_t speed);					//  
};


// --------------------------------------- TRailwayButtonsControl --------------------------------------------------
// 

// 
#define BTN1	0
#define	BTN2	1
#define BTN3	2

class TRailwayButtonsControl {
public: // 
	uint8_t					m_On[8];		//  
protected: // 
	TInOutExtenderS*		m_pIo;			//   
	unsigned long			m_PrevTm;		//   
	uint8_t					m_Counter[8];	//   
public: // 
	TRailwayButtonsControl(void);
	void Setup(TInOutExtenderS* io);
	void LoopCall(void);					//  
};




// ------------------------------------------ TRailwayHardware -----------------------------------------------------
//    

class TRailwayHardware {
public: // 
	TInOutExtenderS			m_Io;			//    -
	TRailwayLightsControl	m_LightsCtrl;	//  
	TRailwayCrossSensors	m_CrossSens;	//  
	TRailwaySwitchControl	m_SwCtrl;		//  
	TRailwayTrainControl	m_TrainCtrl;	//  
	TRailwayButtonsControl	m_BtnCtrl;		// 
public: // 
	TRailwayHardware(void);
	void Setup(void);						//    setup
	void LoopCall(void);					//     loop
};




#endif
