#include <InOutExtender.h>
#include <AFMotor.h>
#include <RailwayHardware.h>
#include <RailwayController.h>

TRailwayHardware HW;
TRailwayController CTRL;

// глобальное состояние (позиция)
static int g_Pos  = 0;

// константы для звуков
const unsigned char SND_VAGONS          = 0; // стук вагонов
const unsigned char SND_SHORT_HORN      = 1; // короткий гудок
const unsigned char SND_LONG_HORN       = 2; // длинный гудок


// Самотестирование
void SelfTest(void) {

  // звуки
  CTRL.PlaySound( SND_VAGONS );
  CTRL.PlaySound( SND_SHORT_HORN );
  CTRL.PlaySound( SND_LONG_HORN );
  
  CTRL.LightsRed( LT1 | LT2 | LT3 | LT4 | LT5 | LT6 );
  CTRL.DelayMs( 500 );
  CTRL.LightsGreen( LT1 | LT2 | LT3 | LT4 | LT5 | LT6 );
  CTRL.DelayMs( 500 );

  CTRL.LightsRed( LT1 | LT2 | LT3 | LT4 | LT5 | LT6 );
  CTRL.LightsGreen( LT1 );
  CTRL.DelayMs( 500 ); 
  CTRL.LightsGreen( LT2 );
  CTRL.DelayMs( 500 ); 
  CTRL.LightsGreen( LT3 );
  CTRL.DelayMs( 500 ); 
  CTRL.LightsGreen( LT4 );
  CTRL.DelayMs( 500 ); 
  CTRL.LightsGreen( LT5 );
  CTRL.DelayMs( 500 ); 
  CTRL.LightsGreen( LT6 );
  CTRL.DelayMs( 500 ); 
  CTRL.LightsRed( LT1 );
  CTRL.DelayMs( 500 ); 
  CTRL.LightsRed( LT2 );
  CTRL.DelayMs( 500 ); 
  CTRL.LightsRed( LT3 );
  CTRL.DelayMs( 500 ); 
  CTRL.LightsRed( LT4 );
  CTRL.DelayMs( 500 ); 
  CTRL.LightsRed( LT5 );
  CTRL.DelayMs( 500 ); 
  CTRL.LightsRed( LT6 );
  CTRL.DelayMs( 500 );
  // стрелка
  CTRL.SwitchRight(1);
  CTRL.DelayMs( 1000 );
  CTRL.SwitchLeft(1);
  CTRL.DelayMs( 1000 );
  // поезд и датчики
  CTRL.LightsGreen( LT1 | LT2 | LT3 | LT4 | LT5 | LT6 );
  CTRL.TrainRun( 6 );
  CTRL.DelayMs( 20000 );
  CTRL.WaitCrossOn( SE1 );
  CTRL.LightsRed( LT5 | LT6 );
  CTRL.TrainRun( 3 );
  CTRL.WaitCrossOn( SE2 );
  CTRL.LightsRed( LT1 );
  CTRL.TrainRun( 1 );
  CTRL.DelayMs( 1000 );
  CTRL.LightsRed( LT2 );
  CTRL.WaitCrossOn( SE3 );
  CTRL.LightsRed( LT3 | LT4 );
  CTRL.DelayMs( 3000 );
  CTRL.TrainRun( 0 );
  
  g_Pos = 2; // остановились в этой позициии
}


void StartSound(void) {
  CTRL.PlaySound( SND_SHORT_HORN );
  CTRL.DelayMs( 1500 );
  CTRL.PlaySound( SND_VAGONS );
}



void StartFromPos1(void) {
  CTRL.LightsGreen( LT1 | LT2 | LT3 | LT4 | LT5 | LT6 );
  StartSound();
  CTRL.TrainRun( 6 );
}

void StartFromPos2(void) {
  CTRL.LightsGreen( LT1 | LT2 | LT3 | LT4 | LT5 | LT6 );
  StartSound();
  CTRL.TrainRun( 6 );
}

void StartFromPos3(void) {
  // move back
  StartSound();
  CTRL.TrainRun( -1 );
  CTRL.WaitCrossOn( SE1 );
  CTRL.DelayMs( 5000 );
  CTRL.TrainRun( 0 );
  CTRL.PlaySound( SND_VAGONS );
  // switch and wait
  CTRL.SwitchLeft(1);
  CTRL.DelayMs( 1000 );
  // go
  CTRL.LightsGreen( LT1 | LT2 | LT3 | LT4 | LT5 | LT6 );
  StartSound();
  CTRL.TrainRun( 6 );
}

void FreeLoops(void) {
  CTRL.WaitCrossOn( SE1 );
  CTRL.DelayMs(1000);
  CTRL.WaitCrossOn( SE1 );
  CTRL.PlaySound( SND_LONG_HORN );
  CTRL.DelayMs(1000);
  CTRL.WaitCrossOn( SE1 );
  CTRL.DelayMs(1000);
}

void StopAtPos1(void) {
  CTRL.WaitCrossOn( SE3 );
  CTRL.LightsRed( LT2 | LT3 | LT4 );
  CTRL.TrainRun( 3 );
  CTRL.DelayMs( 2000 );
  CTRL.LightsRed( LT5 );
  CTRL.WaitCrossOn( SE1 );
  CTRL.LightsRed( LT1 | LT6 );
  CTRL.TrainRun( 1 );
  CTRL.WaitCrossOn( SE2 );
  CTRL.TrainRun( 0 );
  CTRL.PlaySound( SND_VAGONS );
}

void StopAtPos2(void) {
  CTRL.WaitCrossOn( SE1 );
  CTRL.LightsRed( LT5 | LT6 );
  CTRL.TrainRun( 3 );
  CTRL.WaitCrossOn( SE2 );
  CTRL.LightsRed( LT1 );
  CTRL.TrainRun( 1 );
  CTRL.DelayMs( 1000 );
  CTRL.LightsRed( LT2 );
  CTRL.WaitCrossOn( SE3 );
  CTRL.LightsRed( LT3 | LT4 );
  CTRL.DelayMs( 3000 );
  CTRL.TrainRun( 0 );
  CTRL.PlaySound( SND_VAGONS );
}

void StopAtPos3(void) {
  CTRL.WaitCrossOn( SE2 );
  CTRL.TrainRun( 3 );
  CTRL.SwitchRight(0);
  
  CTRL.WaitCrossOn( SE3 );
  CTRL.LightsRed( LT2 | LT3 | LT4 );
  CTRL.TrainRun( 1 );
  CTRL.DelayMs( 4000 );
  CTRL.LightsRed( LT5 );

  CTRL.WaitCrossOn( SE1 );
  CTRL.LightsRed( LT1 | LT6 );
  CTRL.DelayMs( 3500 );
  CTRL.TrainRun( 0 );
  CTRL.PlaySound( SND_VAGONS );
}


// поехать в позицию 1
void GoToPos1(void) {
  switch( g_Pos ) {
    case 1: StartFromPos1(); break;
    case 2: StartFromPos2(); break;
    case 3: StartFromPos3(); break;
  }
  FreeLoops();
  StopAtPos1();
  g_Pos = 1;// потом окажемся в этой позиции
}

// поехать в позицию 2
void GoToPos2(void) {
  switch( g_Pos ) {
    case 1: StartFromPos1(); break;
    case 2: StartFromPos2(); break;
    case 3: StartFromPos3(); break;
  }
  FreeLoops();
  StopAtPos2();
  g_Pos = 2;// потом окажемся в этой позиции
}

// поехать в позицию 3
void GoToPos3(void) {
  switch( g_Pos ) {
    case 1: StartFromPos1(); break;
    case 2: StartFromPos2(); break;
    case 3: StartFromPos3(); break;
  }
  FreeLoops();
  StopAtPos3();
  g_Pos = 3;// потом окажемся в этой позиции
}



void setup(void) {
  HW.Setup();
  CTRL.Setup( &HW );
  // начнем с самотестирования
  SelfTest();
}

void loop(void) {
  HW.LoopCall();
  CTRL.LoopCall();
  delay(10);
  
  if( ! CTRL.IsExecuting() ) {

    if( HW.m_BtnCtrl.m_On[ BTN1 ] ) GoToPos1();
    if( HW.m_BtnCtrl.m_On[ BTN2 ] ) GoToPos2();
    if( HW.m_BtnCtrl.m_On[ BTN3 ] ) GoToPos3();
  }
}

